#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const promises_1 = __importDefault(require("node:fs/promises"));
const node_crypto_1 = __importDefault(require("node:crypto"));
const commander_1 = __importDefault(require("commander"));
const util_1 = require("./util");
const main = async () => {
    const program = new commander_1.default.Command()
        .name("modrinth-updater")
        .version("1.0.5")
        .description("A tool to update your mods from Modrinth")
        .option("-m, --modfolder <path>", "The path to your mods folder");
    program.parse(process.argv);
    const modfolder = program.opts().modfolder || (0, util_1.getModsFolderPath)();
    if (!modfolder) {
        (0, util_1.error)("Could not automatically detect your mods folder. Please specify it manually with the --modfolder option.");
        process.exit(1);
    }
    const modlist = await (0, util_1.getOrGenerateConfig)(`${modfolder}/.modlist.json`).catch(() => {
        (0, util_1.error)("Could not parse config file!");
        process.exit(1);
    });
    await (0, util_1.warnForUnstable)(modlist);
    const oldModFolder = (await promises_1.default.readdir(modfolder)).filter(f => f.endsWith(".jar"));
    const temporaryFolder = `${modfolder}/.modrinth-updater-temporary`;
    try {
        await promises_1.default.mkdir(temporaryFolder);
    }
    catch (e) {
        if (e.code !== "EEXIST") {
            (0, util_1.error)("Failed to create temporary folder!");
            process.exit(1);
        }
    }
    const oldModsAndHashes = await Promise.all(oldModFolder.map(async (mod) => {
        const sha256 = node_crypto_1.default.createHash("sha256"), sha1 = node_crypto_1.default.createHash("sha1");
        const file = await promises_1.default.readFile(`${modfolder}/${mod}`);
        sha256.update(file);
        sha1.update(file);
        return [
            mod,
            {
                sha256: sha256.digest("hex"),
                sha1: sha1.digest("hex")
            }
        ];
    })), noRemove = new Set();
    (0, util_1.info)("Beginning download process... \n");
    const downloadProcess = async (mod) => {
        const [name, file] = await (0, util_1.getLatestRelease)(mod, modlist).catch(() => {
            (0, util_1.error)(`Failed to get latest release for ${mod}!`);
            process.exit(1);
        });
        const oldMod = oldModsAndHashes.find(([mod]) => mod === `${name}.jar`);
        if (file.hashes.sha256) {
            if (oldMod && oldMod[1].sha256 === file.hashes.sha256) {
                (0, util_1.info)(`${mod} is already up to date! (sha256)`);
                noRemove.add(oldMod[0]);
                return;
            }
        }
        if (oldMod && oldMod[1].sha1 === file.hashes.sha1) {
            noRemove.add(oldMod[0]);
            (0, util_1.info)(`${mod} is already up to date! (sha1)`);
            return;
        }
        await (0, util_1.downloadFile)(file.url, `${temporaryFolder}/${name}.jar`).catch(() => {
            process.exit(1);
        });
        const localFile = await promises_1.default.readFile(`${temporaryFolder}/${name}.jar`);
        const checkHash = (hash, type) => {
            const hashFunc = node_crypto_1.default.createHash(type);
            hashFunc.update(localFile);
            if (hashFunc.digest("hex") !== hash) {
                if (modlist.unsafe.allowFailHash) {
                    (0, util_1.error)(`Hash mismatch for ${mod}! (${type})`);
                    process.exit(1);
                }
                else {
                    (0, util_1.warn)(`Hash mismatch for ${mod}! (${type}). Will continue anyway. Be careful!`);
                }
            }
        };
        file.hashes.sha256 ? checkHash(file.hashes.sha256, "sha256") : checkHash(file.hashes.sha1, "sha1");
        (0, util_1.success)(`Downloaded ${mod}!`);
    };
    await Promise.all(modlist.mods.map(downloadProcess));
    (0, util_1.success)("Finished downloading all mods!\n", "\n");
    (0, util_1.info)("Removing old mods...");
    for (const mod of oldModsAndHashes) {
        if (!noRemove.has(mod[0]) && !modlist.externalMods.includes(mod[0])) {
            await promises_1.default.unlink(`${modfolder}/${mod[0]}`);
        }
    }
    (0, util_1.success)("Removed old mods!\n");
    (0, util_1.info)("Moving new mods...");
    for (const mod of await promises_1.default.readdir(temporaryFolder)) {
        await promises_1.default.rename(`${temporaryFolder}/${mod}`, `${modfolder}/${mod}`);
    }
    (0, util_1.success)("Moved new mods!\n");
    (0, util_1.info)("Cleaning up...");
    await promises_1.default.rm(temporaryFolder, { recursive: true });
    (0, util_1.success)("Cleaned up!\n");
    (0, util_1.success)("Finished updating mods!");
};
main().catch(util_1.error);
