"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.warnForUnstable = exports.getOrGenerateConfig = exports.getModsFolderPath = exports.downloadFile = exports.getLatestRelease = exports.success = exports.info = exports.warn = exports.error = exports.api = void 0;
const promises_1 = __importDefault(require("node:fs/promises"));
const node_fs_1 = require("node:fs");
const node_fetch_1 = __importDefault(require("node-fetch"));
const types_1 = require("./types");
const chalk_1 = __importDefault(require("chalk"));
const api = (str, queryParams) => {
    if (str.startsWith("/"))
        str = str.slice(1);
    return `https://api.modrinth.com/v2/${str}${queryParams ? "?" + new URLSearchParams(queryParams).toString() : ""}`;
};
exports.api = api;
const error = (msg, prefix) => console.error((prefix ?? "") + chalk_1.default.bold.red("✘ ") + chalk_1.default.red(msg));
exports.error = error;
const warn = (msg, prefix) => console.warn((prefix ?? "") + chalk_1.default.bold.yellow("! ") + chalk_1.default.yellow(msg));
exports.warn = warn;
const info = (msg, prefix) => console.info((prefix ?? "") + chalk_1.default.bold.blue("ℹ ") + chalk_1.default.blue(msg));
exports.info = info;
const success = (msg, prefix) => console.info((prefix ?? "") + chalk_1.default.bold.green("✓ ") + chalk_1.default.green(msg));
exports.success = success;
const getLatestRelease = async (mod, modlist) => {
    const res = await (0, node_fetch_1.default)((0, exports.api)(`/project/${mod}/version`, {
        game_versions: JSON.stringify([modlist.minecraftVersion]),
        loaders: JSON.stringify([modlist.loaderType])
    })), json = (await res.json());
    if (!res.ok) {
        (0, exports.error)(`Failed to fetch mod data for ${mod}!`);
        process.exit(1);
    }
    const parsed = types_1.apiResponseValidator.parse(json);
    const allowed = parsed.filter(v => v.version_type === "release" || modlist.unsafe.allowUnstable);
    if (allowed.length === 0) {
        (0, exports.error)(`No files found for ${mod}!`);
        process.exit(1);
    }
    const latest = allowed[0];
    if (latest.version_type !== "release")
        (0, exports.warn)(`Downloading an unstable version of ${mod}! It's not guaranteed to work, and may cause issues. If your game crashes, the mod is probably not compatible.`);
    return [mod, latest.files.filter(f => f.primary)[0] ?? latest.files[0]];
};
exports.getLatestRelease = getLatestRelease;
const downloadFile = async (url, path) => {
    const res = await (0, node_fetch_1.default)(url);
    if (!res.ok) {
        (0, exports.error)(`Failed to download file from ${url}!`);
        process.exit(1);
    }
    const fileStream = (0, node_fs_1.createWriteStream)(path);
    await new Promise((resolve, reject) => {
        res.body.pipe(fileStream);
        res.body.on("error", reject);
        fileStream.on("finish", resolve);
    });
};
exports.downloadFile = downloadFile;
const getModsFolderPath = () => {
    let dotMinecraftMods = null;
    switch (process.platform) {
        case "win32":
            dotMinecraftMods = `C:\\Users\\${process.env.USERNAME}\\AppData\\Roaming\\.minecraft\\mods`;
            break;
        case "linux":
            dotMinecraftMods = `${process.env.HOME}/.minecraft/mods`;
            break;
        case "darwin":
            dotMinecraftMods = `${process.env.HOME}/Library/Application Support/.minecraft/mods`;
            break;
    }
    return dotMinecraftMods;
};
exports.getModsFolderPath = getModsFolderPath;
const getOrGenerateConfig = async (configPath) => {
    let content;
    try {
        content = await promises_1.default.readFile(configPath, "utf8");
    }
    catch (e) {
        content = `{
			"minecraftVersion": "1.19.3",
			"loaderType": "fabric",
			"unsafe": {
				"allowFailHash": false,
				"allowUnstable": false,
				"allowProprietary": true
			},
			"mods": [],
			"externalMods": []
		}`;
        await promises_1.default.writeFile(configPath, content);
        (0, exports.info)(`Generated default config at ${configPath}`);
    }
    const rawModlist = JSON.parse(content);
    const parseResult = types_1.modlistValidator.safeParse(rawModlist);
    if (!parseResult.success) {
        (0, exports.error)(`Invalid config file!
${parseResult.error}`);
        process.exit(1);
    }
    return parseResult.data;
};
exports.getOrGenerateConfig = getOrGenerateConfig;
const warnForUnstable = async (modlist) => {
    for (const [k] of Object.entries(modlist.unsafe).filter(([, v]) => v === true)) {
        if (k === "allowFailHash") {
            (0, exports.warn)("You have enabled allowFailHash. This means that if a mod's hash doesn't match the one in the config, it will still be downloaded. This is not recommended, as the mod may be corrupted or malicious.");
        }
        else if (k === "allowUnstable") {
            (0, exports.warn)("You have enabled allowUnstable. This means that unstable versions of mods will be downloaded. This is not recommended, as the mod may cause issues and may even crash your game.");
        }
    }
};
exports.warnForUnstable = warnForUnstable;
